function [line, newmask] = ComputeLine(x_0, grad, mask, param)
    % Draws a flow line for grad starting from x_0.
    % mask.continue is there to indicate us where to stop
    % 
    % param :
    %       param.computeline.method = {'cauchy', 'explicit-bilinear',
    %           'explicit-piecewise', 'runge-kutta-23', 'runge-kutta-45'}
    %       param.computeline.ode_options
    %           Useful for controlling error when using runge-kutta
    %       param.computeline.time_step : double
    %           N.B. : typically, around 0.1
    %       param.computeline.direction = {'both', 'forward', 'backward'}
    %
    %       param.computeline.stopping.time.type = {'time', 'arc-length'}
    %           N.B. : this is the exploration time in each direction
    %       param.computeline.stopping.time.val  : double
    %           N.B. : choose a ridiculously large value for non-stopping
    %           lines
    %
    %       param.computeline.stopping.grad_norm_threshold : double
    %           Stops to draw line if the gradient norm at the current point
    %           is below a fixed threshold.
    %
    %       param.mask.continue.filter : correlation matrix that will be
    %           applied to mask.continue around every line point.
    %       param.mask.continue.type = {'multiplicative', 'additive'}
    %           N.B. : binary is a sub-type of multiplicative
    %       param.mask.continue.thresh : double
    %           N.B. : Typically, between 0 and 1
    %      
    %       param.computeline.max_line_size :
    %           Typically around 500, the maximum number of points a line
    %           may contain in each direction.
    %
    % Author : Jean Feydy
    
    line = [];
    [sx, sy] = size(param.mask.continue.filter);
    assert(isequal(sx, sy), 'Continue filter must be a square matrix.');
    assert(isequal(mod(sx, 2), 1), 'Continue filter must be (odd x odd) (--> centered).');
    mask_padding = (sx - 1)/2;
    padded_mask = padarray(mask.continue, [mask_padding, mask_padding]);
    
    [SY, SX, ~] = size(grad);
    
    if or(isequal(param.computeline.direction, 'both'), strcmp(param.computeline.direction, 'forward'))
        line = ComputeLine_aux();
    end;
    if or(isequal(param.computeline.direction, 'both'), strcmp(param.computeline.direction, 'backward'))
        grad = -grad;
        line2 = ComputeLine_aux();
        line = [fliplr(line2), x_0, line];
    end;    
    if strcmp(param.computeline.direction, 'forward')
        line = [x_0, line];
    end;
    
    truc = cellfun(@(x) ApplyContinueMaskOnPaddedArray(ToGrid(x)), num2cell(line, 1), 'UniformOutput', false);
    
    % We do not forget to remove the zero padding of the mask
    newmask = mask;
    newmask.continue = padded_mask(mask_padding + 1:end-mask_padding, mask_padding + 1:end-mask_padding);
   
    
    
    function list = ComputeLine_aux()
        % Advance in a direction, list does not contain x_0
        % This function is typically called two times
        myline = zeros(2, param.computeline.max_line_size);
        
        % Stopping criterions :
        n_points = 0;
        time = 0;
        next_point = x_0;
        
        list = zeros(2, param.computeline.max_line_size); % For optimization purposes, we need to have a maxsize
        
        if   (strcmp(param.computeline.method, 'cauchy') || ...
                strcmp(param.computeline.method, 'explicit-bilinear') || ...
                strcmp(param.computeline.method, 'explicit-piecewise') )
            next_point_int = max(1, ceil(next_point));
            next_point_int = min(next_point_int, [SY; SX]);
            %ApplyContinueMaskOnPaddedArray(next_point_int); % This is not done by GetNewStartingPoint, so we have to do it now
            
            while 1
                try 
                    [next_point, gr, dt] = GetNextPoint(next_point, grad, param);
                    % N.B. : 'dt = ds' if param.computeline.stopping.time.type =
                    % 'arclength'

                    n_points = n_points + 1;
                    time = time + dt;
                    list(:, n_points) = next_point;

                    % We need to work with integer indices for masking purposes
                    next_point_int = max(1, ceil(next_point));
                    next_point_int = min(next_point_int, [SX; SY]);

                    % We update the mask we will send back
                    %ApplyContinueMaskOnPaddedArray(next_point_int);
                    % But we use the former mask as long as the current line is
                    % concerned : we do not want to prevent ourselves from
                    % plotting...
                catch ME
                    if strcmp(ME.identifier, 'GetNextPoint:StopLine')
                        fprintf('5');
                        break;
                    else
                        rethrow(ME)
                    end;
                end;
                if mask.continue(next_point_int(2), next_point_int(1)) < param.mask.continue.thresh
                    %t = mask.continue(next_point_int(2)-4:next_point_int(2)+4, next_point_int(1)-4:next_point_int(1)+4)
                    fprintf('1');
                    break;
                elseif time >= param.computeline.stopping.time.val
                    fprintf('2');
                    break;
                elseif param.computeline.stopping.grad_norm_threshold >= ...
                        norm(gr)
                    fprintf('3');
                    break;
                elseif n_points >= param.computeline.max_line_size
                    fprintf('4');
                    break;
                end;
            end;
            list = list(:, 1:n_points);
        elseif (strcmp(param.computeline.method, 'runge-kutta-23') || ...
                strcmp(param.computeline.method, 'runge-kutta-45') )
            switch param.computeline.method
                case 'runge-kutta-23'
                    solver = @ode23;
                case 'runge-kutta-45'
                    solver = @ode45;
            end;
            fun = @(t,x) GetGradientFromFloatPoint(x, grad, param);
            STEP = 10;
            while 1
                [t, y] = solver(fun, [0, STEP], next_point, param.computeline.ode_options);
                t = t';
                y = y';
                % We remove the first point, so that we do not count it two
                % times
                t = t(2:end);
                y = y(:, 2:end);
                
                n_newpoints = numel(t);
                n_toadd = min(param.computeline.max_line_size - n_points, n_newpoints);
                list(:, n_points + 1 : n_points + n_toadd) = y(:,1:n_toadd);
                n_points = n_points + n_toadd;
                
                if strcmp(param.computeline.stopping.time.type, 'arc-length')
                    % We compute the arclength of the new segment
                    di = y(:, 2:end) - y(:, 1:end-1);
                    d = sqrt(di(1,:).^2 + di(2,:).^2);
                    time = time + sum(d);
                elseif strcmp(param.computeline.stopping.time.type, 'time')
                    time = time + STEP;
                end;
                
                % next_point is the last point of the curve part we
                % calculated
                next_point = y(:, n_newpoints);
                next_point_int = max(1, ceil(next_point));
                next_point_int = min(next_point_int, [SX; SY]);

                if mask.continue(next_point_int(2), next_point_int(1)) < param.mask.continue.thresh
                    fprintf('1');
                    break;
                elseif time >= param.computeline.stopping.time.val
                    fprintf('2');
                    break;
                elseif param.computeline.stopping.grad_norm_threshold >= ...
                        norm(GetGradientFromFloatPoint(next_point, grad, param))
                    fprintf('3');
                    break;
                elseif n_points >= param.computeline.max_line_size
                    fprintf('4');
                    break;
                end;
                
            end;
            list = list(:, 1:n_points);
            %truc = cellfun(@(x) ApplyContinueMaskOnPaddedArray(ToGrid(x)), num2cell(list, 1), 'UniformOutput', false);
        end;
    end
    function x_i = ToGrid(x_f)
        % Maps a float point to the grid, in order to simply get values
        % such as grad.
        %
        % Author : Jean Feydy
        x_i = max(1, ceil(x_f));
        x_i = min(x_i, [SX; SY]);
    end
    function [f] = ApplyContinueMaskOnPaddedArray(x)
        f = 0;
        % We apply the continue filter around the point x
        if isequal(param.mask.continue.type, 'multiplicative')
            padded_mask(x(2):x(2)+ 2*mask_padding, x(1):x(1)+ 2*mask_padding) = ...
                padded_mask(x(2):x(2)+ 2*mask_padding, x(1):x(1)+ 2*mask_padding) .* param.mask.continue.filter;
        elseif isequal(param.mask.continue.type, 'additive')
            padded_mask(x(2):x(2)+ 2*mask_padding, x(1):x(1)+ 2*mask_padding) = ...
                padded_mask(x(2):x(2)+ 2*mask_padding, x(1):x(1)+ 2*mask_padding) + param.mask.continue.filter;
        else
            error('param.mask.continue.type must be either multiplicative or additive');
        end;
    end

end
