function gr = GetGradientFromFloatPoint(x, grad, param)
    % Sends back the estimated gradient from matrix grad, at point ~x,
    % using param.
    %       param.grad.interpolation_method : {'piecewise', 'bilinear'}
    %           N.B. in bilinear case, pixels are understood to be centered
    %           around (x-0.5, y-0.5)
    %
    % Author : Jean Feydy
    
    switch param.grad.interpolation_method
        case 'piecewise'
            x_i = ToGrid(x);
            gr = grad(x_i(2), x_i(1), :);
        case 'bilinear'
            [x_tl, x_br, tx, ty] = Pixel4Neighborhood(x);
            gr_tl = grad(x_tl(2), x_tl(1), :);
            gr_tr = grad(x_tl(2), x_br(1), :);
            gr_bl = grad(x_br(2), x_tl(1), :);
            gr_br = grad(x_br(2), x_br(1), :);
            gr = (1-ty)*( (1-tx)*gr_tl + tx*gr_tr) + ...
                    ty *( (1-tx)*gr_bl + tx*gr_br);
    end;
    gr = permute(gr, [3,1,2]);
    
    function x_i = ToGrid(x_f)
        % Maps a float point to the grid, in order to simply get values
        % such as grad.
        %
        % Author : Jean Feydy
        [SY, SX, ~] = size(grad);
        x_i = max(1, ceil(x_f));
        x_i = min(x_i, [SX; SY]);
    end
    
    function [x_tl, x_br, tx, ty] = Pixel4Neighborhood(x)
        % Returns x_topleft, x_bottomright, time_x, time_y such that :
        %   (with y axis upside-down, as usual)
        %
        %   x_tl ---tx-------- o
        %    |                 |
        %    ty      x         |
        %    |                 |
        %    |                 |
        %    o   ------------ x_br
        %
        %  i.e. : 
        %       [1;1] <= x_tl <= x <= x_br <= [SX, SY]
        %                                  <= x_tl + [1;1]
        %       0 <= tx,ty < 1
        %       
        %       x = x_tl + [tx;ty]
        %
        % + exceptions if we are on the borders of the frame
        %
        % Author : Jean Feydy
        
        [SY, SX, ~] = size(grad);
        x = StayInFrame(x);
        
        x_tl = max(1, floor(x + 0.5) );
        x_br = min([SX;SY], ceil(x + 0.5));
        t = min(1, max(0, x + 0.5 - x_tl));
        tx = t(1);
        ty = t(2);
        
        function x_b = StayInFrame(x)
            x_b = max(0, min(x, [SX;SY]));
        end
    end
end