function ret = ProcessGradient(grad, param)
    % ret is a post-processed version of grad, according to param.
    % param :
    %       param.grad.blur : correlation filter ?
    %       param.grad.processing_method : {'none', 'correlation',
    %               'coherent-line-drawing', 'median'}
    %       param.grad.median_size : [h w]
    %       param.grad.pre_rotation : double (degree)
    %       param.grad.post_rotation: double (degree)
    %           Use those two parameters to rotate the vector field :
    %               using 90 will typically end up with ~contour lines
    %
    % Author : Jean Feydy
    
    grad = RotateField(grad, param.grad.pre_rotation);
    switch param.grad.processing_method
        case 'none'
            ret = grad;
        case 'correlation'
            CorrelationFilterGradient();
        case 'coherent-line-drawing'
            CoherentLineDrawingGradient(); 
        case 'median'
            MedianFilterGradient();
        otherwise
            error('param.grad.processing_method has illegal value');
    end;
    
    ret = RotateField(ret, param.grad.post_rotation);
    
    function r = RotateField(f, theta)
        % Rotate a nxmx2 vector field (typically, grad) by the angle theta,
        % in radians.
        %
        % Author : Jean Feydy
        
        c = cosd(theta);
        s = sind(theta);
        r(:,:,1) = c * f(:,:,1) + s * f(:,:,2);
        r(:,:,2) = -s * f(:,:,1) + c * f(:,:,2);
    end
    
    function CorrelationFilterGradient()
        % Compute the gradient by applying param.grad.blur as a filter
        %
        % Author : Jean Feydy
        ret(:,:,1) = imfilter(grad(:,:,1), param.grad.blur, 'replicate');
        ret(:,:,2) = imfilter(grad(:,:,2), param.grad.blur, 'replicate');
    end

    function MedianFilterGradient()
            % Compute the gradient by applying param.grad.median_size as a filter
            %
            % Author : Jean Feydy
            ret(:,:,1) = medfilt2(grad(:,:,1), param.grad.median_size, 'symmetric');
            ret(:,:,2) = medfilt2(grad(:,:,2), param.grad.median_size, 'symmetric');
    end
    function CoherentLineDrawingGradient()
        % TO DO
    end
end